@@ -7,7 +7,7 @@ reset_sub = function(x) {
 	
 	for (i in xy) {
 		if (d[[ i ]]$from > 1) {
-			if (!is_regular(x))
+			if (!is_regular_grid(x))
 				stop("can only write sub-rasters for regular grids")
 			ioff = d[[ i ]]$from - 1
 			d[[ i ]]$offset = d[[ i ]]$offset + ioff * d[[ i ]]$delta
@@ -33,21 +33,25 @@ write_stars = function(obj, dsn, layer, ...) UseMethod("write_stars")
 #' @param obj object of class \code{stars}
 #' @param dsn gdal dataset (file) name
 #' @param layer attribute name; if missing, the first attribute is written
-#' @param ... passed on to \link[sf]{gdal_write}
+#' @param ... passed on to \link[sf:gdal]{gdal_write}
 #' @param driver driver driver name; see \link[sf]{st_drivers}
 #' @param options character vector with options
 #' @param type character; output binary type, one of: \code{Byte} for eight bit unsigned integer, \code{UInt16} for sixteen bit unsigned integer, \code{Int16} for sixteen bit signed integer, \code{UInt32} for thirty two bit unsigned integer, \code{Int32} for thirty two bit signed integer, \code{Float32} for thirty two bit floating point, \code{Float64} for sixty four bit floating point.
 #' @param NA_value non-NA value that should represent R's \code{NA} value in the target raster file; if set to \code{NA}, it will be ignored.
 #' @param update logical; if \code{TRUE}, an existing file is being updated
+#' @param normalize_path logical; see \link{read_stars}
 #' @name write_stars
 #' @export
 write_stars.stars = function(obj, dsn, layer = 1, ..., driver = detect.driver(dsn), 
-		options = character(0), type = "Float32", NA_value = NA_real_, update = FALSE) {
-	if (length(obj) > 1 && missing(layer))
+		options = character(0), type = "Float32", NA_value = NA_real_, update = FALSE,
+		normalize_path = TRUE) {
+	if (missing(layer) && length(obj) > 1)
 		warning("all but first attribute are ignored")
 	obj = st_upfront(obj[layer])
 	if (! update) # new file: should not be a sub-array
 		obj = reset_sub(obj)
+	if (normalize_path)
+		dsn = enc2utf8(maybe_normalizePath(dsn, TRUE))
 	sf::gdal_write(obj, ..., file = dsn, driver = driver, options = options, 
 		type = type, NA_value = NA_value, geotransform = get_geotransform(obj), 
 		update = update)
@@ -63,16 +67,31 @@ write_stars.stars_proxy = function(obj, dsn, layer = 1, ..., driver = detect.dri
 		options = character(0), type = "Float32", NA_value = NA_real_, 
 		chunk_size = c(dim(obj)[1], floor(25e6 / dim(obj)[1])), progress = TRUE) {
 
-	if (length(obj) > 1 && missing(layer))
+	if (missing(layer) && length(obj) > 1)
 		warning("all but first attribute are ignored")
+	if (layer != 1)
+		stop("only first attribute of a stars_proxy object can be written; consider using merge")
+	if (length(obj[[1]]) > 1) { # collapse bands:
+		out_file = tempfile(fileext = ".vrt")
+		gdal_utils("buildvrt", x[[1]], out_file, options = "-separate")
+		x[[1]] = out_file
+	}
 	if (progress) {
 		pb = txtProgressBar()
 		setTxtProgressBar(pb, 0)
 	}
 
-	# write chunks:
 	dim_obj = dim(obj)
+	if (prod(chunk_size) > prod(dim_obj[1:2])) {
+		write_stars(st_as_stars(obj), dsn, layer, ..., driver = driver, options = options,
+			type = type, NA_value = NA_value)
+		return(invisible(obj))
+	}
+
+	# write chunks:
 	di = st_dimensions(obj)
+	if (di[[1]]$from > 1 || di[[2]]$from > 1)
+		message("chunked writing may not work for subsetted rasters: in case of failure use write_stars(st_as_stars(object))")
 
 	created = FALSE
 
@@ -107,8 +126,13 @@ write_stars.stars_proxy = function(obj, dsn, layer = 1, ..., driver = detect.dri
 	invisible(obj)
 }
 
-#nocov start
-detect.driver = function(filename) {
+#' @name write_stars
+#' @export
+#' @param filename character; used for guessing driver short name based on file 
+#' extension; see examples
+#' @examples 
+#' detect.driver("L7_ETMs.tif")
+detect.driver = function(filename) { #nocov start
 	# from raster::.getFormat:
 	ext <- tolower(tools::file_ext(filename))
 	if (nchar(ext) < 2) {
@@ -118,7 +142,7 @@ detect.driver = function(filename) {
 		if (ext == 'tif' | ext == 'tiff') { return('GTiff')
 		} else if (ext == 'grd') { return('raster')
 		} else if (ext == 'asc') { return('ascii')
-		} else if (ext == 'nc' | ext == 'cdf' | ext == 'ncdf') { return('CDF')
+		} else if (ext == 'nc' || ext == 'cdf' || ext == 'ncdf') { return('netcdf')
 		} else if (ext == 'kml') { return('KML')
 		} else if (ext == 'kmz') { return('KML')		
 		} else if (ext == 'big') { return('big.matrix')
